@@ -6,7 +6,21 @@ set -x
 CFLAGS_FOR_OVS="-g -O2"
 SPARSE_FLAGS=""
 EXTRA_OPTS="--enable-Werror"
-TARGET="x86_64-native-linuxapp-gcc"
+
+on_exit() {
+    if [ $? = 0 ]; then
+        exit
+    fi
+    FILES_TO_PRINT="config.log"
+    FILES_TO_PRINT="$FILES_TO_PRINT */_build/sub/tests/testsuite.log"
+
+    for pr_file in $FILES_TO_PRINT; do
+        cat "$pr_file" 2>/dev/null
+    done
+}
+# We capture the error logs as artifacts in Github Actions, no need to dump
+# them via a EXIT handler.
+[ -n "$GITHUB_WORKFLOW" ] || trap on_exit EXIT
 
 function install_kernel()
 {
@@ -88,6 +102,28 @@ function install_dpdk()
 {
     local DPDK_VER=$1
     local VERSION_FILE="dpdk-dir/travis-dpdk-cache-version"
+    local DPDK_OPTS=""
+    local DPDK_LIB=""
+
+    if [ -z "$TRAVIS_ARCH" ] ||
+       [ "$TRAVIS_ARCH" == "amd64" ]; then
+        DPDK_LIB=$(pwd)/dpdk-dir/build/lib/x86_64-linux-gnu
+    elif [ "$TRAVIS_ARCH" == "aarch64" ]; then
+        DPDK_LIB=$(pwd)/dpdk-dir/build/lib/aarch64-linux-gnu
+    else
+        echo "Target is unknown"
+        exit 1
+    fi
+
+    if [ "$DPDK_SHARED" ]; then
+        EXTRA_OPTS="$EXTRA_OPTS --with-dpdk=shared"
+        export LD_LIBRARY_PATH=$DPDK_LIB/:$LD_LIBRARY_PATH
+    else
+        EXTRA_OPTS="$EXTRA_OPTS --with-dpdk=static"
+    fi
+
+    # Export the following path for pkg-config to find the .pc file.
+    export PKG_CONFIG_PATH=$DPDK_LIB/pkgconfig/:$PKG_CONFIG_PATH
 
     if [ "${DPDK_VER##refs/*/}" != "${DPDK_VER}" ]; then
         # Avoid using cache for git tree build.
@@ -101,7 +137,8 @@ function install_dpdk()
         if [ -f "${VERSION_FILE}" ]; then
             VER=$(cat ${VERSION_FILE})
             if [ "${VER}" = "${DPDK_VER}" ]; then
-                EXTRA_OPTS="${EXTRA_OPTS} --with-dpdk=$(pwd)/dpdk-dir/build"
+                # Update the library paths.
+                sudo ldconfig
                 echo "Found cached DPDK ${VER} build in $(pwd)/dpdk-dir"
                 return
             fi
@@ -115,27 +152,24 @@ function install_dpdk()
         pushd dpdk-dir
     fi
 
-    make config CC=gcc T=$TARGET
+    # Switching to 'default' machine to make dpdk-dir cache usable on
+    # different CPUs. We can't be sure that all CI machines are exactly same.
+    DPDK_OPTS="$DPDK_OPTS -Dmachine=default"
 
-    if [ "$DPDK_SHARED" ]; then
-        sed -i '/CONFIG_RTE_BUILD_SHARED_LIB=n/s/=n/=y/' build/.config
-        export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$(pwd)/$TARGET/lib
-    fi
+    # Disable building DPDK unit tests. Not needed for OVS build or tests.
+    DPDK_OPTS="$DPDK_OPTS -Dtests=false"
 
-    # Disable building DPDK kernel modules. Not needed for OVS build or tests.
-    sed -i '/CONFIG_RTE_EAL_IGB_UIO=y/s/=y/=n/' build/.config
-    sed -i '/CONFIG_RTE_KNI_KMOD=y/s/=y/=n/' build/.config
+    # Install DPDK using prefix.
+    DPDK_OPTS="$DPDK_OPTS --prefix=$(pwd)/build"
 
-    # Enable pdump support in DPDK.
-    sed -i '/CONFIG_RTE_LIBRTE_PMD_PCAP=n/s/=n/=y/' build/.config
-    sed -i '/CONFIG_RTE_LIBRTE_PDUMP=n/s/=n/=y/' build/.config
+    CC=gcc meson $DPDK_OPTS build
+    ninja -C build
+    ninja -C build install
+
+    # Update the library paths.
+    sudo ldconfig
 
-    # Switching to 'default' machine to make dpdk-dir cache usable on different
-    # CPUs.  We can't be sure that all CI machines are exactly same.
-    sed -i '/CONFIG_RTE_MACHINE="native"/s/="native"/="default"/' build/.config
 
-    make -j4 CC=gcc EXTRA_CFLAGS='-fPIC'
-    EXTRA_OPTS="$EXTRA_OPTS --with-dpdk=$(pwd)/build"
     echo "Installed DPDK source in $(pwd)"
     popd
     echo "${DPDK_VER}" > ${VERSION_FILE}
@@ -144,7 +178,7 @@ function install_dpdk()
 function configure_ovs()
 {
     ./boot.sh
-    ./configure CFLAGS="${CFLAGS_FOR_OVS}" $* || { cat config.log; exit 1; }
+    ./configure CFLAGS="${CFLAGS_FOR_OVS}" $*
 }
 
 function build_ovs()
@@ -161,7 +195,7 @@ function build_ovs()
         make -j4
         popd
     else
-        make -j4 || { cat config.log; exit 1; }
+        make -j4
     fi
 }
 
@@ -182,11 +216,9 @@ fi
 
 if [ "$DPDK" ] || [ "$DPDK_SHARED" ]; then
     if [ -z "$DPDK_VER" ]; then
-        DPDK_VER="19.11.10"
+        DPDK_VER="20.11.1"
     fi
     install_dpdk $DPDK_VER
-    # Enable pdump support in OVS.
-    EXTRA_OPTS="${EXTRA_OPTS} --enable-dpdk-pdump"
     if [ "$CC" = "clang" ]; then
         # Disregard cast alignment errors until DPDK is fixed
         CFLAGS_FOR_OVS="${CFLAGS_FOR_OVS} -Wno-cast-align"
@@ -200,7 +232,7 @@ elif [ "$M32" ]; then
     # Adding m32 flag directly to CC to avoid any posiible issues with API/ABI
     # difference on 'configure' and 'make' stages.
     export CC="$CC -m32"
-else
+elif [ "$TRAVIS_ARCH" != "aarch64" ]; then
     OPTS="--enable-sparse"
     if [ "$AFXDP" ]; then
         # netdev-afxdp uses memset for 64M for umem initialization.
@@ -209,6 +241,15 @@ else
     CFLAGS_FOR_OVS="${CFLAGS_FOR_OVS} ${SPARSE_FLAGS}"
 fi
 
+if [ "$ASAN" ]; then
+    # This will override default option configured in tests/atlocal.in.
+    export ASAN_OPTIONS='detect_leaks=1'
+    # -O2 generates few false-positive memory leak reports in test-ovsdb
+    # application, so lowering optimizations to -O1 here.
+    CLFAGS_ASAN="-O1 -fno-omit-frame-pointer -fno-common -fsanitize=address"
+    CFLAGS_FOR_OVS="${CFLAGS_FOR_OVS} ${CLFAGS_ASAN}"
+fi
+
 save_OPTS="${OPTS} $*"
 OPTS="${EXTRA_OPTS} ${save_OPTS}"
 
@@ -218,12 +259,8 @@ if [ "$TESTSUITE" ]; then
     configure_ovs
 
     export DISTCHECK_CONFIGURE_FLAGS="$OPTS"
-    if ! make distcheck -j4 CFLAGS="${CFLAGS_FOR_OVS}" \
-         TESTSUITEFLAGS=-j4 RECHECK=yes; then
-        # testsuite.log is necessary for debugging.
-        cat */_build/sub/tests/testsuite.log
-        exit 1
-    fi
+    make distcheck -j4 CFLAGS="${CFLAGS_FOR_OVS}" \
+        TESTSUITEFLAGS=-j4 RECHECK=yes
 else
     if [ -z "${KERNEL_LIST}" ]; then build_ovs ${KERNEL};
     else
