@@ -1112,9 +1112,9 @@ OvsPopFieldInPacketBuf(OvsForwardingContext *ovsFwdCtx,
      * should split the function and refactor. */
     if (!bufferData) {
         EthHdr *ethHdr = (EthHdr *)bufferStart;
+        /* If the frame is not VLAN make it a no op */
         if (ethHdr->Type != ETH_TYPE_802_1PQ_NBO) {
-            OVS_LOG_ERROR("Invalid ethHdr type %u, nbl %p", ethHdr->Type, ovsFwdCtx->curNbl);
-            return NDIS_STATUS_INVALID_PACKET;
+            return NDIS_STATUS_SUCCESS;
         }
     }
     RtlMoveMemory(bufferStart + shiftLength, bufferStart, shiftOffset);
@@ -1137,9 +1137,6 @@ OvsPopFieldInPacketBuf(OvsForwardingContext *ovsFwdCtx,
 static __inline NDIS_STATUS
 OvsPopVlanInPktBuf(OvsForwardingContext *ovsFwdCtx)
 {
-    NDIS_STATUS status;
-    OVS_PACKET_HDR_INFO* layers = &ovsFwdCtx->layers;
-
     /*
      * Declare a dummy vlanTag structure since we need to compute the size
      * of shiftLength. The NDIS one is a unionized structure.
@@ -1148,15 +1145,7 @@ OvsPopVlanInPktBuf(OvsForwardingContext *ovsFwdCtx)
     UINT32 shiftLength = sizeof(vlanTag.TagHeader);
     UINT32 shiftOffset = sizeof(DL_EUI48) + sizeof(DL_EUI48);
 
-    status = OvsPopFieldInPacketBuf(ovsFwdCtx, shiftOffset, shiftLength,
-                                    NULL);
-
-    if (status == NDIS_STATUS_SUCCESS) {
-        layers->l3Offset -= (UINT16) shiftLength;
-        layers->l4Offset -= (UINT16) shiftLength;
-    }
-
-    return status;
+    return OvsPopFieldInPacketBuf(ovsFwdCtx, shiftOffset, shiftLength, NULL);
 }
 
 
@@ -1803,11 +1792,9 @@ OvsExecuteRecirc(OvsForwardingContext *ovsFwdCtx,
     }
 
     if (newNbl) {
-        deferredAction = OvsAddDeferredActions(newNbl, key, &(ovsFwdCtx->layers),
-                                               NULL);
+        deferredAction = OvsAddDeferredActions(newNbl, key, NULL);
     } else {
-        deferredAction = OvsAddDeferredActions(ovsFwdCtx->curNbl, key,
-                                              &(ovsFwdCtx->layers), NULL);
+        deferredAction = OvsAddDeferredActions(ovsFwdCtx->curNbl, key, NULL);
     }
 
     if (deferredAction) {
@@ -1977,7 +1964,7 @@ OvsExecuteSampleAction(OvsForwardingContext *ovsFwdCtx,
         return STATUS_SUCCESS;
     }
 
-    if (!OvsAddDeferredActions(newNbl, key, &(ovsFwdCtx->layers), a)) {
+    if (!OvsAddDeferredActions(newNbl, key, a)) {
         OVS_LOG_INFO(
             "Deferred actions limit reached, dropping sample action.");
         OvsCompleteNBL(ovsFwdCtx->switchContext, newNbl, TRUE);
@@ -2113,7 +2100,6 @@ OvsDoExecuteActions(POVS_SWITCH_CONTEXT switchContext,
                  */
                 status = OvsPopVlanInPktBuf(&ovsFwdCtx);
                 if (status != NDIS_STATUS_SUCCESS) {
-                    OVS_LOG_ERROR("OVS-pop vlan action failed status = %lu", status);
                     dropReason = L"OVS-pop vlan action failed";
                     goto dropit;
                 }
@@ -2363,7 +2349,7 @@ OvsActionsExecute(POVS_SWITCH_CONTEXT switchContext,
 
     if (status == STATUS_SUCCESS) {
         status = OvsProcessDeferredActions(switchContext, completionList,
-                                           portNo, sendFlags, NULL);
+                                           portNo, sendFlags, layers);
     }
 
     return status;
