@@ -22,13 +22,12 @@
 #include <sys/stat.h>
 #include <getopt.h>
 
+#include <rte_cpuflags.h>
 #include <rte_errno.h>
 #include <rte_log.h>
+#include <rte_malloc.h>
 #include <rte_memzone.h>
 #include <rte_version.h>
-#ifdef DPDK_PDUMP
-#include <rte_pdump.h>
-#endif
 
 #include "dirs.h"
 #include "fatal-signal.h"
@@ -39,6 +38,7 @@
 #include "ovs-numa.h"
 #include "smap.h"
 #include "svec.h"
+#include "unixctl.h"
 #include "util.h"
 #include "vswitch-idl.h"
 
@@ -130,22 +130,63 @@ construct_dpdk_options(const struct smap *ovs_other_config, struct svec *args)
     }
 }
 
+static int
+compare_numa_node_list(const void *a_, const void *b_)
+{
+    int a = *(const int *) a_;
+    int b = *(const int *) b_;
+
+    if (a < b) {
+        return -1;
+    }
+    if (a > b) {
+        return 1;
+    }
+    return 0;
+}
+
 static char *
 construct_dpdk_socket_mem(void)
 {
     const char *def_value = "1024";
-    int numa, numa_nodes = ovs_numa_get_n_numas();
     struct ds dpdk_socket_mem = DS_EMPTY_INITIALIZER;
 
-    if (numa_nodes == 0 || numa_nodes == OVS_NUMA_UNSPEC) {
-        numa_nodes = 1;
-    }
+    /* Build a list of all numa nodes with at least one core. */
+    struct ovs_numa_dump *dump = ovs_numa_dump_n_cores_per_numa(1);
+    size_t n_numa_nodes = hmap_count(&dump->numas);
+    int *numa_node_list = xcalloc(n_numa_nodes, sizeof *numa_node_list);
 
-    ds_put_cstr(&dpdk_socket_mem, def_value);
-    for (numa = 1; numa < numa_nodes; ++numa) {
-        ds_put_format(&dpdk_socket_mem, ",%s", def_value);
-    }
+    const struct ovs_numa_info_numa *node;
+    int k = 0, last_node = 0;
 
+    FOR_EACH_NUMA_ON_DUMP(node, dump) {
+        if (k >= n_numa_nodes) {
+            break;
+        }
+        numa_node_list[k++] = node->numa_id;
+    }
+    qsort(numa_node_list, k, sizeof *numa_node_list, compare_numa_node_list);
+
+    for (int i = 0; i < n_numa_nodes; i++) {
+        while (numa_node_list[i] > last_node &&
+               numa_node_list[i] != OVS_NUMA_UNSPEC &&
+               numa_node_list[i] <= MAX_NUMA_NODES) {
+            if (last_node == 0) {
+                ds_put_format(&dpdk_socket_mem, "%s", "0");
+            } else {
+                ds_put_format(&dpdk_socket_mem, ",%s", "0");
+            }
+            last_node++;
+        }
+        if (numa_node_list[i] == 0) {
+            ds_put_format(&dpdk_socket_mem, "%s", def_value);
+        } else {
+            ds_put_format(&dpdk_socket_mem, ",%s", def_value);
+        }
+        last_node++;
+    }
+    free(numa_node_list);
+    ovs_numa_dump_destroy(dump);
     return ds_cstr(&dpdk_socket_mem);
 }
 
@@ -176,6 +217,7 @@ construct_dpdk_mutex_options(const struct smap *ovs_other_config,
         int found_opts = 0, scan, found_pos = -1;
         const char *found_value;
         struct dpdk_exclusive_options_map *popt = &excl_opts[i];
+        bool using_default = false;
 
         for (scan = 0; scan < MAX_DPDK_EXCL_OPTS
                  && popt->ovs_dpdk_options[scan]; ++scan) {
@@ -192,6 +234,7 @@ construct_dpdk_mutex_options(const struct smap *ovs_other_config,
             if (popt->default_option) {
                 found_pos = popt->default_option;
                 found_value = popt->default_value;
+                using_default = true;
             } else {
                 continue;
             }
@@ -204,6 +247,12 @@ construct_dpdk_mutex_options(const struct smap *ovs_other_config,
         }
 
         if (!args_contains(args, popt->eal_dpdk_options[found_pos])) {
+            if (using_default) {
+                VLOG_INFO("Using default value for '%s'. OVS will no longer "
+                          "provide a default for this argument starting "
+                          "from 2.17 release. DPDK defaults will be used "
+                          "instead.", popt->eal_dpdk_options[found_pos]);
+            }
             svec_add(args, popt->eal_dpdk_options[found_pos]);
             svec_add(args, found_value);
         } else {
@@ -264,6 +313,105 @@ static cookie_io_functions_t dpdk_log_func = {
     .write = dpdk_log_write,
 };
 
+static void
+dpdk_unixctl_mem_stream(struct unixctl_conn *conn, int argc OVS_UNUSED,
+                        const char *argv[] OVS_UNUSED, void *aux)
+{
+    void (*callback)(FILE *) = aux;
+    char *response = NULL;
+    FILE *stream;
+    size_t size;
+
+    stream = open_memstream(&response, &size);
+    if (!stream) {
+        response = xasprintf("Unable to open memstream: %s.",
+                             ovs_strerror(errno));
+        unixctl_command_reply_error(conn, response);
+        goto out;
+    }
+
+    callback(stream);
+    fclose(stream);
+    unixctl_command_reply(conn, response);
+out:
+    free(response);
+}
+
+static int
+dpdk_parse_log_level(const char *s)
+{
+    static const char * const levels[] = {
+        [RTE_LOG_EMERG]   = "emergency",
+        [RTE_LOG_ALERT]   = "alert",
+        [RTE_LOG_CRIT]    = "critical",
+        [RTE_LOG_ERR]     = "error",
+        [RTE_LOG_WARNING] = "warning",
+        [RTE_LOG_NOTICE]  = "notice",
+        [RTE_LOG_INFO]    = "info",
+        [RTE_LOG_DEBUG]   = "debug",
+    };
+    int i;
+
+    for (i = 1; i < ARRAY_SIZE(levels); ++i) {
+        if (!strcmp(s, levels[i])) {
+            return i;
+        }
+    }
+    return -1;
+}
+
+static void
+dpdk_unixctl_log_set(struct unixctl_conn *conn, int argc, const char *argv[],
+                     void *aux OVS_UNUSED)
+{
+    int i;
+
+    /* With no argument, set all components level to 'debug'. */
+    if (argc == 1) {
+        rte_log_set_level_pattern("*", RTE_LOG_DEBUG);
+    }
+    for (i = 1; i < argc; i++) {
+        char *err_msg = NULL;
+        char *level_string;
+        char *pattern;
+        char *s;
+        int level;
+
+        s = xstrdup(argv[i]);
+        level_string = strchr(s, ':');
+        if (level_string == NULL) {
+            pattern = "*";
+            level_string = s;
+        } else {
+            pattern = s;
+            level_string[0] = '\0';
+            level_string++;
+        }
+
+        level = dpdk_parse_log_level(level_string);
+        if (level == -1) {
+            err_msg = xasprintf("invalid log level: '%s'", level_string);
+        } else if (rte_log_set_level_pattern(pattern, level) < 0) {
+            err_msg = xasprintf("cannot set log level for '%s'", argv[i]);
+        }
+
+        if (err_msg) {
+            unixctl_command_reply_error(conn, err_msg);
+            free(err_msg);
+            free(s);
+            return;
+        }
+        free(s);
+    }
+    unixctl_command_reply(conn, NULL);
+}
+
+static void
+malloc_dump_stats_wrapper(FILE *stream)
+{
+    rte_malloc_dump_stats(stream, NULL);
+}
+
 static bool
 dpdk_init__(const struct smap *ovs_other_config)
 {
@@ -342,6 +490,10 @@ dpdk_init__(const struct smap *ovs_other_config)
         if (i < args.n - 1) {
             svec_add(&args, "--socket-limit");
             svec_add(&args, args.names[i + 1]);
+            VLOG_INFO("Using default value for '--socket-limit'. OVS will no "
+                      "longer provide a default for this argument starting "
+                      "from 2.17 release. DPDK defaults will be used "
+                      "instead.");
         }
     }
 
@@ -351,7 +503,7 @@ dpdk_init__(const struct smap *ovs_other_config)
 
     /**
      * NOTE: This is an unsophisticated mechanism for determining the DPDK
-     * lcore for the DPDK Master.
+     * main core.
      */
     if (auto_determine) {
         const struct ovs_numa_info_core *core;
@@ -416,30 +568,30 @@ dpdk_init__(const struct smap *ovs_other_config)
         FILE *stream = open_memstream(&response, &size);
 
         if (stream) {
+            fprintf(stream, "rte_memzone_dump:\n");
             rte_memzone_dump(stream);
+            fprintf(stream, "rte_log_dump:\n");
+            rte_log_dump(stream);
             fclose(stream);
-            if (size) {
-                VLOG_DBG("rte_memzone_dump:\n%s", response);
-            }
+            VLOG_DBG("%s", response);
             free(response);
         } else {
-            VLOG_DBG("Could not dump memzone. Unable to open memstream: %s.",
-                     ovs_strerror(errno));
+            VLOG_DBG("Could not dump memzone and log levels. "
+                     "Unable to open memstream: %s.", ovs_strerror(errno));
         }
     }
 
+    unixctl_command_register("dpdk/log-list", "", 0, 0,
+                             dpdk_unixctl_mem_stream, rte_log_dump);
+    unixctl_command_register("dpdk/log-set", "{level | pattern:level}", 0,
+                             INT_MAX, dpdk_unixctl_log_set, NULL);
+    unixctl_command_register("dpdk/get-malloc-stats", "", 0, 0,
+                             dpdk_unixctl_mem_stream,
+                             malloc_dump_stats_wrapper);
+
     /* We are called from the main thread here */
     RTE_PER_LCORE(_lcore_id) = NON_PMD_CORE_ID;
 
-#ifdef DPDK_PDUMP
-    VLOG_WARN("DPDK pdump support is deprecated and "
-              "will be removed in next OVS releases.");
-    err = rte_pdump_init();
-    if (err) {
-        VLOG_INFO("Error initialising DPDK pdump");
-    }
-#endif
-
     /* Finally, register the dpdk classes */
     netdev_dpdk_register();
     netdev_register_flow_api_provider(&netdev_offload_dpdk);
@@ -525,6 +677,58 @@ print_dpdk_version(void)
     puts(rte_version());
 }
 
+/* Avoid calling rte_cpu_get_flag_enabled() excessively, by caching the
+ * result of the call for each CPU flag in a static variable. To avoid
+ * allocating large numbers of static variables, use a uint8 as a bitfield.
+ * Note the macro must only return if the ISA check is done and available.
+ */
+#define ISA_CHECK_DONE_BIT (1 << 0)
+#define ISA_AVAILABLE_BIT  (1 << 1)
+
+#define CHECK_CPU_FEATURE(feature, name_str, RTE_CPUFLAG)               \
+    do {                                                                \
+        if (strncmp(feature, name_str, strlen(name_str)) == 0) {        \
+            static uint8_t isa_check_##RTE_CPUFLAG;                     \
+            int check = isa_check_##RTE_CPUFLAG & ISA_CHECK_DONE_BIT;   \
+            if (OVS_UNLIKELY(!check)) {                                 \
+                int has_isa = rte_cpu_get_flag_enabled(RTE_CPUFLAG);    \
+                VLOG_DBG("CPU flag %s, available %s\n",                 \
+                         name_str, has_isa ? "yes" : "no");             \
+                isa_check_##RTE_CPUFLAG = ISA_CHECK_DONE_BIT;           \
+                if (has_isa) {                                          \
+                    isa_check_##RTE_CPUFLAG |= ISA_AVAILABLE_BIT;       \
+                }                                                       \
+            }                                                           \
+            if (isa_check_##RTE_CPUFLAG & ISA_AVAILABLE_BIT) {          \
+                return true;                                            \
+            } else {                                                    \
+                return false;                                           \
+            }                                                           \
+        }                                                               \
+    } while (0)
+
+bool
+dpdk_get_cpu_has_isa(const char *arch, const char *feature)
+{
+    /* Ensure Arch is x86_64. */
+    if (strncmp(arch, "x86_64", 6) != 0) {
+        return false;
+    }
+
+#if __x86_64__
+    /* CPU flags only defined for the architecture that support it. */
+    CHECK_CPU_FEATURE(feature, "avx512f", RTE_CPUFLAG_AVX512F);
+    CHECK_CPU_FEATURE(feature, "avx512bw", RTE_CPUFLAG_AVX512BW);
+    CHECK_CPU_FEATURE(feature, "avx512vbmi", RTE_CPUFLAG_AVX512VBMI);
+    CHECK_CPU_FEATURE(feature, "avx512vpopcntdq", RTE_CPUFLAG_AVX512VPOPCNTDQ);
+    CHECK_CPU_FEATURE(feature, "bmi2", RTE_CPUFLAG_BMI2);
+#endif
+
+    VLOG_WARN("Unknown CPU arch,feature: %s,%s. Returning not supported.\n",
+              arch, feature);
+    return false;
+}
+
 void
 dpdk_status(const struct ovsrec_open_vswitch *cfg)
 {
