@@ -38,6 +38,7 @@ OVSDB_UPDATE = 0
 OVSDB_UPDATE2 = 1
 
 CLUSTERED = "clustered"
+RELAY = "relay"
 
 
 Notice = collections.namedtuple('Notice', ('event', 'row', 'updates'))
@@ -491,6 +492,15 @@ class Idl(object):
         :type updates:  Row
         """
 
+    def cooperative_yield(self):
+        """Hook for cooperatively yielding to eventlet/gevent/asyncio/etc.
+
+        When a block of code is going to spend a lot of time cpu-bound without
+        doing any I/O, it can cause greenthread/coroutine libraries to block.
+        This call should be added to code where this can happen, but defaults
+        to doing nothing to avoid overhead where it is not needed.
+        """
+
     def __send_cond_change(self, table, cond):
         monitor_cond_change = {table.name: [{"where": cond}]}
         old_uuid = str(self.uuid)
@@ -656,6 +666,8 @@ class Idl(object):
                                       'is not an object'
                                       % (table_name, uuid_string))
 
+                self.cooperative_yield()
+
                 if version == OVSDB_UPDATE2:
                     changes = self.__process_update2(table, uuid, row_update)
                     if changes:
@@ -789,8 +801,7 @@ class Idl(object):
                       % (session_name, self._db.name))
             return False
 
-        if (database.model == CLUSTERED and
-            self._session.get_num_of_remotes() > 1):
+        if database.model == CLUSTERED:
             if not database.schema:
                 vlog.info('%s: clustered database server has not yet joined '
                           'cluster; trying another server' % session_name)
@@ -810,6 +821,21 @@ class Idl(object):
                               'trying another server' % session_name)
                     return False
                 self._min_index = database.index[0]
+        elif database.model == RELAY:
+            if not database.schema:
+                vlog.info('%s: relay database server has not yet connected '
+                          'to the relay source; trying another server'
+                          % session_name)
+                return False
+            if not database.connected:
+                vlog.info('%s: relay database server is disconnected '
+                          'from the relay source; trying another server'
+                          % session_name)
+                return False
+            if self.leader_only:
+                vlog.info('%s: relay database server cannot be a leader; '
+                          'trying another server' % session_name)
+                return False
 
         return True
 
@@ -1479,11 +1505,6 @@ class Transaction(object):
         if self != self.idl.txn:
             return self._status
 
-        if self.idl.state != Idl.IDL_S_MONITORING:
-            self._status = Transaction.TRY_AGAIN
-            self.__disassemble()
-            return self._status
-
         # If we need a lock but don't have it, give up quickly.
         if self.idl.lock_name and not self.idl.has_lock:
             self._status = Transaction.NOT_LOCKED
